import websocket
import threading
import time
import ssl
import base64
import json
from pydub import AudioSegment
import paho.mqtt.client as mqtt
import hmac
import hashlib
from urllib.parse import urlparse, urlencode, quote_plus
from datetime import datetime
import pytz

# 常量定义
API_KEY = 'd5a68918d375de6a88b95594f1fd7e55'
API_SECRET = 'NzExNjFhNWM2ZTczZTI5NjZhNjQ3N2My'
APP_ID = '9d181e31'
AUDIO_FILE = 'open.wav'  # 音频文件路径
MQTT_HOST = '127.0.0.1'
MQTT_TOPIC = 'stm32'
MQTT_PORT = 1883
MQTT_KEEPALIVE_INTERVAL = 60  # Keepalive interval in seconds
# 创建 MQTT 客户端实例
client = mqtt.Client(mqtt.CallbackAPIVersion.VERSION1)
# 科大讯飞 WebSocket URL
HOST = 'wss://iat-api.xfyun.cn/v2/iat'


def assemble_auth_url(hosturl, api_key, api_secret):
    try:
        # 解析 URL
        parsed_url = urlparse(hosturl)
        host = parsed_url.hostname
        path = parsed_url.path

        # 生成签名时间
        date = datetime.now(pytz.utc).strftime('%a, %d %b %Y %H:%M:%S GMT')

        # 参与签名的字段
        sign_string = f"host: {host}\ndate: {date}\nGET {path} HTTP/1.1"

        # 计算签名
        signature = hmac.new(api_secret.encode('utf-8'), sign_string.encode('utf-8'), hashlib.sha256).digest()
        encoded_signature = base64.b64encode(signature).decode('utf-8')

        # 构建请求参数
        auth_url = f'api_key="{api_key}", algorithm="hmac-sha256", headers="host date request-line", signature="{encoded_signature}"'
        authorization = base64.b64encode(auth_url.encode('utf-8')).decode('utf-8')

        # 构建完整的 URL
        params = {
            'host': host,
            'date': date,
            'authorization': authorization
        }
        encoded_params = urlencode(params, quote_via=quote_plus)
        call_url = f"{hosturl}?{encoded_params}"

        return call_url
    except Exception as e:
        print(f"Error assembling auth URL: {e}")

def check_and_convert_audio(file_path):
    audio = AudioSegment.from_file(file_path)
    sample_rate = audio.frame_rate
    bits_per_sample = audio.sample_width * 8

    # 检查采样率和位深
    if sample_rate not in [8000, 16000] or bits_per_sample != 16:
        # 转换音频文件
        audio = audio.set_frame_rate(16000).set_sample_width(2)
        output_path = file_path.rsplit('.', 1)[0] + "_converted.wav"
        audio.export(output_path, format="wav")
        return output_path
    return file_path

def on_message(ws, message):
    print("Received:", message)
    response = json.loads(message)
    if response.get("message") == "success":
        data = response.get("data", {})
        result = data.get("result", {}).get("ws", [])
        text = "".join([w["cw"][0]["w"] for w in result])
        print("识别结果:", text)
        if "开" in text:
             send_command('1')
            #print("发送的命令变量:", '1')
        elif "关" in text:
            send_command('2')
            #print("发送的命令变量:", '2')
        else:
            send_command('0')
            #print("发送的命令变量:", '0')

def on_error(ws, error):
    print("Error:", error)

def on_close(ws, close_status_code, close_msg):
    print("### closed ###")

def on_open(ws,processed_file):
    def run(*args):
        frame = {
            "common": {"app_id": APP_ID},
            "business": {
                "language": "zh_cn",
                "domain": "iat",
                "accent": "mandarin",
                "dwa": "wpgs"  # 动态修正，以便捕捉到更多词语
            },
            "data": {
                "status": 0,
                "format": "audio/L16;rate=16000",
                "encoding": "raw"
            }
        }
        # 发送音频数据
        with open(processed_file, "rb") as f:
            frame['data']['status'] = 0  # 开始
            data = f.read(1280)
            while data:
                frame['data']['audio'] = base64.b64encode(data).decode('utf-8')
                ws.send(json.dumps(frame))
                frame['data']['status'] = 1  # 中间
                data = f.read(1280)
                time.sleep(0.04)
            frame['data']['status'] = 2  # 结束
            ws.send(json.dumps(frame))
        time.sleep(1)
        ws.close()
    threading.Thread(target=run).start()

# 连接到 MQTT 服务器
def on_connect(client, userdata, flags, rc):
    print("Connected with result code "+str(rc))

def send_variable(var):
    # 转换命令为字符串
    message = str(var)
    # 发布消息到指定的主题
    client.publish(MQTT_TOPIC, message)
    print(f"Sent command '{message}' to topic {MQTT_TOPIC}")

def send_command(command):
    # 在连接事件后被调用
    client.on_connect = on_connect
    # 连接到服务器
    client.connect(MQTT_HOST, MQTT_PORT, MQTT_KEEPALIVE_INTERVAL)
    # 开始 MQTT 客户端网络循环
    client.loop_start()
    # 发送变量
    send_variable(command)
    # 断开连接并停止网络循环
    time.sleep(2)  # 留时间让消息发送出去
    client.loop_stop()
    client.disconnect()

# 脚本主逻辑
if __name__ == "__main__":
    websocket.enableTrace(True)
    processed_file = check_and_convert_audio(AUDIO_FILE)
    if processed_file:
        host=assemble_auth_url(HOST, API_KEY, API_SECRET)
        ws = websocket.WebSocketApp(host,
                                    on_open=lambda ws: on_open(ws, processed_file),
                                    on_message=on_message,
                                    on_error=on_error,
                                    on_close=on_close,
                                    header={"Authorization": "Bearer " + API_KEY})   # 认证方式需调整为有效方式
        #ws.on_open = on_open
        ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})  # 生产环境中不应禁用 SSL 验证
    #    send_command(command)
       # print(f"Command sent: {command}")
    else:
        print("Failed to process audio file.")
